@@ -75,6 +75,7 @@ void php_http_client_options_set_subr(zval *instance, char *key, size_t len, zva
 
 		array_init(&new_opts);
 		old_opts = zend_read_property(this_ce, instance, ZEND_STRL("options"), 0, &old_opts_tmp);
+
 		if (Z_TYPE_P(old_opts) == IS_ARRAY) {
 			array_copy(Z_ARRVAL_P(old_opts), Z_ARRVAL(new_opts));
 		}
@@ -88,6 +89,7 @@ void php_http_client_options_set_subr(zval *instance, char *key, size_t len, zva
 			}
 		} else if (opts && zend_hash_num_elements(Z_ARRVAL_P(opts))) {
 			if ((entry = zend_symtable_str_find(Z_ARRVAL(new_opts), key, len))) {
+				SEPARATE_ZVAL(entry);
 				array_join(Z_ARRVAL_P(opts), Z_ARRVAL_P(entry), 0, 0);
 			} else {
 				Z_ADDREF_P(opts);
@@ -687,7 +689,7 @@ static PHP_METHOD(HttpClient, enqueue)
 	zend_fcall_info_cache fcc = empty_fcall_info_cache;
 	php_http_client_object_t *obj;
 	php_http_message_object_t *msg_obj;
-	php_http_client_enqueue_t q;
+	php_http_client_enqueue_t q = {0};
 
 	php_http_expect(SUCCESS == zend_parse_parameters(ZEND_NUM_ARGS(), "O|f", &request, php_http_get_client_request_class_entry(), &fci, &fcc), invalid_arg, return);
 
@@ -699,6 +701,17 @@ static PHP_METHOD(HttpClient, enqueue)
 		return;
 	}
 
+	/* set early for progress callback */
+	q.opaque = msg_obj;
+
+	if (obj->client->callback.progress.func) {
+		php_http_client_progress_state_t progress = {0};
+
+		progress.info = "prepare";
+		obj->client->callback.progress.func(obj->client->callback.progress.arg, obj->client, &q, &progress);
+	}
+
+	Z_ADDREF_P(request);
 	q.request = msg_obj->message;
 	q.options = combined_options(getThis(), request);
 	q.dtor = msg_queue_dtor;
@@ -709,12 +722,10 @@ static PHP_METHOD(HttpClient, enqueue)
 	if (fci.size) {
 		Z_TRY_ADDREF(fci.function_name);
 		if (fci.object) {
-			++GC_REFCOUNT(fci.object);
+			GC_ADDREF(fci.object);
 		}
 	}
 
-	Z_ADDREF_P(request);
-
 	php_http_expect(SUCCESS == php_http_client_enqueue(obj->client, &q), runtime,
 			msg_queue_dtor(&q);
 			return;
@@ -779,7 +790,7 @@ static PHP_METHOD(HttpClient, requeue)
 	if (fci.size) {
 		Z_TRY_ADDREF(fci.function_name);
 		if (fci.object) {
-			++GC_REFCOUNT(fci.object);
+			GC_ADDREF(fci.object);
 		}
 	}
 
