@@ -21,7 +21,7 @@
 #	include <openssl/ssl.h>
 #endif
 #if PHP_HTTP_HAVE_LIBCURL_GNUTLS
-#	include <gnutls.h>
+#	include <gnutls/gnutls.h>
 #endif
 
 typedef struct php_http_client_curl_handler {
@@ -218,6 +218,7 @@ static int php_http_curle_raw_callback(CURL *ch, curl_infotype type, char *data,
 	switch (type) {
 		case CURLINFO_TEXT:
 			if (data[0] == '-') {
+				goto text;
 			} else if (php_memnstr(data, ZEND_STRL("Adding handle:"), data + length)) {
 				h->progress.info = "setup";
 			} else if (php_memnstr(data, ZEND_STRL("addHandle"), data + length)) {
@@ -234,8 +235,16 @@ static int php_http_curle_raw_callback(CURL *ch, curl_infotype type, char *data,
 				h->progress.info = "connected";
 			} else if (php_memnstr(data, ZEND_STRL("blacklisted"), data + length)) {
 				h->progress.info = "blacklist check";
+			} else if (php_memnstr(data, ZEND_STRL("TLS"), data + length)) {
+				h->progress.info = "ssl negotiation";
 			} else if (php_memnstr(data, ZEND_STRL("SSL"), data + length)) {
 				h->progress.info = "ssl negotiation";
+			} else if (php_memnstr(data, ZEND_STRL("certificate"), data + length)) {
+				h->progress.info = "ssl negotiation";
+			} else if (php_memnstr(data, ZEND_STRL("ALPN"), data + length)) {
+				h->progress.info = "alpn";
+			} else if (php_memnstr(data, ZEND_STRL("NPN"), data + length)) {
+				h->progress.info = "npn";
 			} else if (php_memnstr(data, ZEND_STRL("upload"), data + length)) {
 				h->progress.info = "uploaded";
 			} else if (php_memnstr(data, ZEND_STRL("left intact"), data + length)) {
@@ -247,6 +256,7 @@ static int php_http_curle_raw_callback(CURL *ch, curl_infotype type, char *data,
 			} else if (php_memnstr(data, ZEND_STRL("Operation timed out"), data + length)) {
 				h->progress.info = "timeout";
 			} else {
+				text:;
 #if 0
 				h->progress.info = data;
 				data[length - 1] = '\0';
@@ -480,6 +490,24 @@ static ZEND_RESULT_CODE php_http_curle_get_info(CURL *ch, HashTable *info)
 		zend_hash_str_update(info, "http_version", lenof("http_version"), &tmp);
 	}
 #endif
+#if PHP_HTTP_CURL_VERSION(7,52,0)
+	if (CURLE_OK == curl_easy_getinfo(ch, CURLINFO_PROXY_SSL_VERIFYRESULT, &l)) {
+		ZVAL_LONG(&tmp, l);
+		zend_hash_str_update(info, "proxy_ssl_verifyresult", lenof("proxy_ssl_verifyresult"), &tmp);
+	}
+#endif
+#if PHP_HTTP_CURL_VERSION(7,52,0)
+	if (CURLE_OK == curl_easy_getinfo(ch, CURLINFO_PROTOCOL, &l)) {
+		ZVAL_LONG(&tmp, l);
+		zend_hash_str_update(info, "protocol", lenof("protocol"), &tmp);
+	}
+#endif
+#if PHP_HTTP_CURL_VERSION(7,52,0)
+	if (CURLE_OK == curl_easy_getinfo(ch, CURLINFO_SCHEME, &c)) {
+		ZVAL_STRING(&tmp, STR_PTR(c));
+		zend_hash_str_update(info, "scheme", lenof("scheme"), &tmp);
+	}
+#endif
 
 	/* END::CURLINFO */
 
@@ -2206,7 +2234,7 @@ static ZEND_RESULT_CODE php_http_client_curl_dequeue(php_http_client_t *h, php_h
 	php_http_client_curl_t *curl = h->ctx;
 	php_http_client_curl_handler_t *handler = enqueue->opaque;
 
-	if (h->callback.depth) {
+	if (h->callback.depth && !CG(unclean_shutdown)) {
 		php_error_docref(NULL, E_WARNING, "Could not dequeue request while executing callbacks");
 		return FAILURE;
 	}
@@ -2429,6 +2457,14 @@ php_http_client_ops_t *php_http_client_curl_get_ops(void)
 	return &php_http_client_curl_ops;
 }
 
+#define REGISTER_NS_STRING_OR_NULL_CONSTANT(ns, name, str, flags)                              \
+		do {                                                                           \
+			if ((str) != NULL) {                                                   \
+				REGISTER_NS_STRING_CONSTANT(ns, name, str, flags);             \
+			} else {                                                               \
+				REGISTER_NS_NULL_CONSTANT(ns, name, flags);                    \
+			}                                                                      \
+		} while (0)
 
 PHP_MINIT_FUNCTION(http_client_curl)
 {
@@ -2509,12 +2545,12 @@ PHP_MINIT_FUNCTION(http_client_curl)
 		REGISTER_NS_STRING_CONSTANT("http\\Client\\Curl", "VERSIONS", curl_version(), CONST_CS|CONST_PERSISTENT);
 #if CURLVERSION_NOW >= 0
 		REGISTER_NS_STRING_CONSTANT("http\\Client\\Curl\\Versions", "CURL", (char *) info->version, CONST_CS|CONST_PERSISTENT);
-		REGISTER_NS_STRING_CONSTANT("http\\Client\\Curl\\Versions", "SSL", (char *) info->ssl_version, CONST_CS|CONST_PERSISTENT);
-		REGISTER_NS_STRING_CONSTANT("http\\Client\\Curl\\Versions", "LIBZ", (char *) info->libz_version, CONST_CS|CONST_PERSISTENT);
+		REGISTER_NS_STRING_OR_NULL_CONSTANT("http\\Client\\Curl\\Versions", "SSL", (char *) info->ssl_version, CONST_CS|CONST_PERSISTENT);
+		REGISTER_NS_STRING_OR_NULL_CONSTANT("http\\Client\\Curl\\Versions", "LIBZ", (char *) info->libz_version, CONST_CS|CONST_PERSISTENT);
 # if CURLVERSION_NOW >= 1
-		REGISTER_NS_STRING_CONSTANT("http\\Client\\Curl\\Versions", "ARES", (char *) info->ares, CONST_CS|CONST_PERSISTENT);
+		REGISTER_NS_STRING_OR_NULL_CONSTANT("http\\Client\\Curl\\Versions", "ARES", (char *) info->ares, CONST_CS|CONST_PERSISTENT);
 #  if CURLVERSION_NOW >= 2
-		REGISTER_NS_STRING_CONSTANT("http\\Client\\Curl\\Versions", "IDN", (char *) info->libidn, CONST_CS|CONST_PERSISTENT);
+		REGISTER_NS_STRING_OR_NULL_CONSTANT("http\\Client\\Curl\\Versions", "IDN", (char *) info->libidn, CONST_CS|CONST_PERSISTENT);
 #  endif
 # endif
 #endif
